@@ -1,8 +1,8 @@
 /*
  *  server.c
- * 
+ *
  *  Copyright (c) 2006 by Miklos Vajna <vmiklos@frugalware.org>
- * 
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -14,22 +14,20 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
- *  USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <stdlib.h>
+#include <errno.h>
+#include <time.h>
 #include <string.h>
+#include <limits.h>
 #include <stdio.h>
-#include <libintl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <time.h>
-#include <errno.h>
 #include <download.h>
 
 /* libalpm */
@@ -40,7 +38,6 @@
 #include "alpm.h"
 #include "util.h"
 #include "handle.h"
-#include "log.h"
 #include "package.h"
 
 pmserver_t *_alpm_server_new(const char *url)
@@ -50,20 +47,15 @@ pmserver_t *_alpm_server_new(const char *url)
 
 	ALPM_LOG_FUNC;
 
-	server = (pmserver_t *)malloc(sizeof(pmserver_t));
-	if(server == NULL) {
-		_alpm_log(PM_LOG_ERROR, _("malloc failure: could not allocate %d bytes"), sizeof(pmserver_t));
-		RET_ERR(PM_ERR_MEMORY, NULL);
-	}
+	CALLOC(server, 1, sizeof(pmserver_t), RET_ERR(PM_ERR_MEMORY, NULL));
 
-	memset(server, 0, sizeof(pmserver_t));
 	u = downloadParseURL(url);
 	if(!u) {
-		_alpm_log(PM_LOG_ERROR, _("url '%s' is invalid, ignoring"), url);
+		_alpm_log(PM_LOG_ERROR, _("url '%s' is invalid, ignoring\n"), url);
 		RET_ERR(PM_ERR_SERVER_BAD_URL, NULL);
 	}
 	if(strlen(u->scheme) == 0) {
-		_alpm_log(PM_LOG_WARNING, _("url scheme not specified, assuming http"));
+		_alpm_log(PM_LOG_WARNING, _("url scheme not specified, assuming http\n"));
 		strcpy(u->scheme, "http");
 	}
 
@@ -81,10 +73,8 @@ pmserver_t *_alpm_server_new(const char *url)
 	return server;
 }
 
-void _alpm_server_free(void *data)
+void _alpm_server_free(pmserver_t *server)
 {
-	pmserver_t *server = data;
-
 	ALPM_LOG_FUNC;
 
 	if(server == NULL) {
@@ -107,7 +97,8 @@ static char *strip_filename(pmserver_t *server)
 	p = strrchr(server->s_url->doc, '/');
 	if(p && *(++p)) {
 		fname = strdup(p);
-		_alpm_log(PM_LOG_DEBUG, _("stripping '%s' from '%s'"), fname, server->s_url->doc);
+		_alpm_log(PM_LOG_DEBUG, "stripping '%s' from '%s'\n",
+				fname, server->s_url->doc);
 		*p = 0;
 	}
 
@@ -124,52 +115,59 @@ static struct url *url_for_file(pmserver_t *server, const char *filename)
 	int doclen = 0;
 
 	doclen = strlen(server->s_url->doc) + strlen(filename) + 2;
-	doc = calloc(doclen, sizeof(char));
-	if(!doc) {
-		RET_ERR(PM_ERR_MEMORY, NULL);
-	}
+	CALLOC(doc, doclen, sizeof(char), RET_ERR(PM_ERR_MEMORY, NULL));
 
 	snprintf(doc, doclen, "%s/%s", server->s_url->doc, filename);
-	_alpm_log(PM_LOG_DEBUG, "file path: '%s'", doc);
 	ret = downloadMakeURL(server->s_url->scheme,
 												server->s_url->host,
 												server->s_url->port,
 												doc,
 												server->s_url->user,
 												server->s_url->pwd);
-	free(doc);
+	FREE(doc);
 	return(ret);
 }
 
 /*
  * Download a list of files from a list of servers
  *   - if one server fails, we try the next one in the list
+ *   - if *dl_total is non-NULL, then it will be used as the starting
+ *     download amount when TotalDownload is set. It will also be
+ *     set to the final download amount for the calling function to use.
+ *   - totalsize is the total download size for use when TotalDownload
+ *     is set. Use 0 if the total download size is not known.
  *
  * RETURN:  0 for successful download, 1 on error
  */
-int _alpm_downloadfiles(alpm_list_t *servers, const char *localpath, alpm_list_t *files)
+int _alpm_downloadfiles(alpm_list_t *servers, const char *localpath,
+		alpm_list_t *files, int *dl_total, unsigned long totalsize)
 {
-	ALPM_LOG_FUNC;
-	return(_alpm_downloadfiles_forreal(servers, localpath, files, NULL, NULL));
+	return(_alpm_downloadfiles_forreal(servers, localpath, files, 0, NULL,
+				dl_total, totalsize));
 }
 
 /*
  * This is the real downloadfiles, used directly by sync_synctree() to check
  * modtimes on remote files.
- *   - if *mtime1 is non-NULL, then only download files
- *     if they are different than *mtime1.  String should be in the form
- *     "YYYYMMDDHHMMSS" to match the form of ftplib's FtpModDate() function.
- *   - if *mtime2 is non-NULL, then it will be filled with the mtime
- *     of the remote file (from MDTM FTP cmd or Last-Modified HTTP header).
- * 
+ *   - if mtime1 is non-NULL, then only download files if they are different
+ *     than mtime1.
+ *   - if *mtime2 is non-NULL, it will be filled with the mtime of the remote
+ *     file.
+ *   - if *dl_total is non-NULL, then it will be used as the starting
+ *     download amount when TotalDownload is set. It will also be
+ *     set to the final download amount for the calling function to use.
+ *   - totalsize is the total download size for use when TotalDownload
+ *     is set. Use 0 if the total download size is not known.
+ *
  * RETURN:  0 for successful download
  *          1 if the mtimes are identical
  *         -1 on error
  */
 int _alpm_downloadfiles_forreal(alpm_list_t *servers, const char *localpath,
-	alpm_list_t *files, const char *mtime1, char *mtime2)
+	alpm_list_t *files, time_t mtime1, time_t *mtime2, int *dl_total,
+	unsigned long totalsize)
 {
-	int dltotal_bytes = 0;
+	int dl_thisfile = 0;
 	alpm_list_t *lp;
 	int done = 0;
 	alpm_list_t *complete = NULL;
@@ -191,23 +189,15 @@ int _alpm_downloadfiles_forreal(alpm_list_t *servers, const char *localpath,
 			char output[PATH_MAX];
 			char *fn = (char *)lp->data;
 			char pkgname[PKG_NAME_LEN];
-			char *p;
 
 			fileurl = url_for_file(server, fn);
 			if(!fileurl) {
 				return(-1);
 			}
 
-			/* Try to get JUST the name of the package from the filename */
-			memset(pkgname, 0, PKG_NAME_LEN);
-			if((p = strstr(fn, PM_EXT_PKG))) {
-				_alpm_pkg_splitname(fn, pkgname, NULL, 1);
-			}
-			if(!strlen(pkgname)) {
-				/* just use the raw filename if we can't find crap */
-				STRNCPY(pkgname, fn, PKG_NAME_LEN+1);
-			}
-			_alpm_log(PM_LOG_DEBUG, _("using '%s' for download progress"), pkgname);
+			/* pass the raw filename for passing to the callback function */
+			strncpy(pkgname, fn, PKG_NAME_LEN);
+			_alpm_log(PM_LOG_DEBUG, "using '%s' for download progress\n", pkgname);
 
 			snprintf(realfile, PATH_MAX, "%s%s", localpath, fn);
 			snprintf(output, PATH_MAX, "%s%s.part", localpath, fn);
@@ -223,66 +213,64 @@ int _alpm_downloadfiles_forreal(alpm_list_t *servers, const char *localpath,
 				int chk_resume = 0;
 
 				if(stat(output, &st) == 0 && st.st_size > 0) {
-					_alpm_log(PM_LOG_DEBUG, _("existing file found, using it"));
+					_alpm_log(PM_LOG_DEBUG, "existing file found, using it\n");
 					fileurl->offset = (off_t)st.st_size;
-					dltotal_bytes = st.st_size;
+					dl_thisfile = st.st_size;
+					if (dl_total != NULL) {
+						*dl_total += st.st_size;
+					}
 					localf = fopen(output, "a");
 					chk_resume = 1;
 				} else {
 					fileurl->offset = (off_t)0;
-					dltotal_bytes = 0;
+					dl_thisfile = 0;
 				}
-				
-				/* libdownload does not reset the error code, reset it in the case of previous errors */
+
+				/* libdownload does not reset the error code, reset it in
+				 * the case of previous errors */
 				downloadLastErrCode = 0;
 
 				/* 10s timeout - TODO make a config option */
 				downloadTimeout = 10000;
 
-			 	/* Make libdownload super verbose... worthwhile for testing */
-				if(alpm_option_get_logmask() & PM_LOG_DOWNLOAD) {
-						downloadDebug = 1;
-				}
-				if(alpm_option_get_logmask() & PM_LOG_DEBUG) {
-						dlf = downloadXGet(fileurl, &ust, (handle->nopassiveftp ? "v" : "vp"));
-				} else {
-						dlf = downloadXGet(fileurl, &ust, (handle->nopassiveftp ? "" : "p"));
-				}
+				dlf = downloadXGet(fileurl, &ust, (handle->nopassiveftp ? "" : "p"));
 
 				if(downloadLastErrCode != 0 || dlf == NULL) {
-					_alpm_log(PM_LOG_ERROR, _("failed retrieving file '%s' from %s : %s"),
-										fn, fileurl->host, downloadLastErrString);
+					const char *host = _("disk");
+					if(strcmp(SCHEME_FILE, fileurl->scheme) != 0) {
+						host = fileurl->host;
+					}
+					_alpm_log(PM_LOG_ERROR, _("failed retrieving file '%s' from %s : %s\n"),
+										fn, host, downloadLastErrString);
 					if(localf != NULL) {
 						fclose(localf);
 					}
 					/* try the next server */
+					downloadFreeURL(fileurl);
 					continue;
 				} else {
-						_alpm_log(PM_LOG_DEBUG, _("connected to %s successfully"), fileurl->host);
+						_alpm_log(PM_LOG_DEBUG, "connected to %s successfully\n", fileurl->host);
 				}
-				
-				if(ust.mtime && mtime1) {
-					char strtime[15];
-					_alpm_time2string(ust.mtime, strtime);
-					if(strcmp(mtime1, strtime) == 0) {
-						_alpm_log(PM_LOG_DEBUG, _("mtimes are identical, skipping %s"), fn);
-						complete = alpm_list_add(complete, fn);
-						if(localf != NULL) {
-							fclose(localf);
-						}
-						if(dlf != NULL) {
-							fclose(dlf);
-						}
-						return(1);
+
+				if(ust.mtime && mtime1 && ust.mtime == mtime1) {
+					_alpm_log(PM_LOG_DEBUG, "mtimes are identical, skipping %s\n", fn);
+					complete = alpm_list_add(complete, fn);
+					if(localf != NULL) {
+						fclose(localf);
 					}
+					if(dlf != NULL) {
+						fclose(dlf);
+					}
+					downloadFreeURL(fileurl);
+					return(1);
 				}
-				
+
 				if(ust.mtime && mtime2) {
-					_alpm_time2string(ust.mtime, mtime2);
+					*mtime2 = ust.mtime;
 				}
 
 				if(chk_resume && fileurl->offset == 0) {
-					_alpm_log(PM_LOG_WARNING, _("cannot resume download, starting over"));
+					_alpm_log(PM_LOG_WARNING, _("cannot resume download, starting over\n"));
 					if(localf != NULL) {
 						fclose(localf);
 						localf = NULL;
@@ -292,51 +280,64 @@ int _alpm_downloadfiles_forreal(alpm_list_t *servers, const char *localpath,
 				if(localf == NULL) {
 					_alpm_rmrf(output);
 					fileurl->offset = (off_t)0;
-					dltotal_bytes = 0;
+					dl_thisfile = 0;
 					localf = fopen(output, "w");
 					if(localf == NULL) { /* still null? */
-						_alpm_log(PM_LOG_ERROR, _("cannot write to file '%s'"), output);
+						_alpm_log(PM_LOG_ERROR, _("cannot write to file '%s'\n"), output);
 						if(dlf != NULL) {
 							fclose(dlf);
 						}
-						return -1;
+						downloadFreeURL(fileurl);
+						return(-1);
 					}
 				}
 
 				/* Progress 0 - initialize */
-				if(handle->dlcb) handle->dlcb(pkgname, 0, ust.size);
+				if(handle->dlcb) {
+					handle->dlcb(pkgname, 0, ust.size, dl_total ? *dl_total : 0,
+							totalsize);
+				}
 
 				int nread = 0;
 				char buffer[PM_DLBUF_LEN];
 				while((nread = fread(buffer, 1, PM_DLBUF_LEN, dlf)) > 0) {
 					if(ferror(dlf)) {
-						_alpm_log(PM_LOG_ERROR, _("error downloading '%s': %s"),
+						_alpm_log(PM_LOG_ERROR, _("error downloading '%s': %s\n"),
 											fn, downloadLastErrString);
 						fclose(localf);
 						fclose(dlf);
+						downloadFreeURL(fileurl);
 						return(-1);
 					}
-					
+
 					int nwritten = 0;
 					while(nwritten < nread) {
 						nwritten += fwrite(buffer, 1, (nread - nwritten), localf);
 						if(ferror(localf)) {
-							_alpm_log(PM_LOG_ERROR, _("error writing to file '%s': %s"),
+							_alpm_log(PM_LOG_ERROR, _("error writing to file '%s': %s\n"),
 												realfile, strerror(errno));
 							fclose(localf);
 							fclose(dlf);
+							downloadFreeURL(fileurl);
 							return(-1);
 						}
 					}
 
 					if(nwritten != nread) {
-						
-					}
-					dltotal_bytes += nread;
 
-					if(handle->dlcb) handle->dlcb(pkgname, dltotal_bytes, ust.size);
+					}
+					dl_thisfile += nread;
+					if (dl_total != NULL) {
+						*dl_total += nread;
+					}
+
+					if(handle->dlcb) {
+						handle->dlcb(pkgname, dl_thisfile, ust.size,
+								dl_total ? *dl_total : 0, totalsize);
+					}
 				}
 
+				downloadFreeURL(fileurl);
 				fclose(localf);
 				fclose(dlf);
 				rename(output, realfile);
@@ -351,7 +352,10 @@ int _alpm_downloadfiles_forreal(alpm_list_t *servers, const char *localpath,
 				char cwd[PATH_MAX];
 
 				/* build the full download url */
-				snprintf(url, PATH_MAX, "%s://%s%s", fileurl->scheme, fileurl->host, fileurl->doc);
+				snprintf(url, PATH_MAX, "%s://%s%s", fileurl->scheme,
+						fileurl->host, fileurl->doc);
+				/* we don't need this anymore */
+				downloadFreeURL(fileurl);
 
 				/* replace all occurrences of %o with fn.part */
 				strncpy(origCmd, handle->xfercommand, sizeof(origCmd));
@@ -378,18 +382,18 @@ int _alpm_downloadfiles_forreal(alpm_list_t *servers, const char *localpath,
 				/* cwd to the download directory */
 				getcwd(cwd, PATH_MAX);
 				if(chdir(localpath)) {
-					_alpm_log(PM_LOG_WARNING, _("could not chdir to %s"), localpath);
+					_alpm_log(PM_LOG_WARNING, _("could not chdir to %s\n"), localpath);
 					return(PM_ERR_CONNECT_FAILED);
 				}
 				/* execute the parsed command via /bin/sh -c */
-				_alpm_log(PM_LOG_DEBUG, _("running command: %s"), parsedCmd);
+				_alpm_log(PM_LOG_DEBUG, "running command: %s\n", parsedCmd);
 				ret = system(parsedCmd);
 				if(ret == -1) {
-					_alpm_log(PM_LOG_WARNING, _("running XferCommand: fork failed!"));
+					_alpm_log(PM_LOG_WARNING, _("running XferCommand: fork failed!\n"));
 					return(PM_ERR_FORK_FAILED);
 				} else if(ret != 0) {
 					/* download failed */
-					_alpm_log(PM_LOG_DEBUG, _("XferCommand command returned non-zero status code (%d)"), ret);
+					_alpm_log(PM_LOG_DEBUG, "XferCommand command returned non-zero status code (%d)\n", ret);
 				} else {
 					/* download was successful */
 					complete = alpm_list_add(complete, fn);
@@ -399,26 +403,36 @@ int _alpm_downloadfiles_forreal(alpm_list_t *servers, const char *localpath,
 				}
 				chdir(cwd);
 			}
-			downloadFreeURL(fileurl);
 		}
 
 		if(alpm_list_count(complete) == alpm_list_count(files)) {
 			done = 1;
 		}
 	}
+	alpm_list_free(complete);
 
 	return(done ? 0 : -1);
 }
 
-char *_alpm_fetch_pkgurl(char *target)
+/** Fetch a remote pkg.
+ * @param url URL of the package to download
+ * @return the downloaded filepath on success, NULL on error
+ * @addtogroup alpm_misc
+ */
+char SYMEXPORT *alpm_fetch_pkgurl(const char *url)
 {
 	pmserver_t *server;
-	char *filename;
-	struct stat st;
+	char *filename, *filepath;
+	const char *cachedir;
 
 	ALPM_LOG_FUNC;
 
-	server = _alpm_server_new(target);
+	if(strstr(url, "://") == NULL) {
+		_alpm_log(PM_LOG_DEBUG, "Invalid URL passed to alpm_fetch_pkgurl\n");
+		return(NULL);
+	}
+
+	server = _alpm_server_new(url);
 	if(!server) {
 		return(NULL);
 	}
@@ -426,30 +440,30 @@ char *_alpm_fetch_pkgurl(char *target)
 	/* strip path information from the filename */
 	filename = strip_filename(server);
 	if(!filename) {
-		_alpm_log(PM_LOG_ERROR, _("URL does not contain a file for download"));
+		_alpm_log(PM_LOG_ERROR, _("URL does not contain a file for download\n"));
 		return(NULL);
 	}
 
-	/* do not download the file if it exists in the current dir */
-	if(stat(filename, &st) == 0) {
-		_alpm_log(PM_LOG_DEBUG, _("%s has already been downloaded"), filename);
-	} else {
-		alpm_list_t *servers = alpm_list_add(NULL, server);
-		alpm_list_t *files = alpm_list_add(NULL, filename);
+	/* find a valid cache dir to download to */
+	cachedir = _alpm_filecache_setup();
 
-		if(_alpm_downloadfiles(servers, "./", files)) {
-			_alpm_log(PM_LOG_WARNING, _("failed to download %s"), target);
-			return(NULL);
-		}
-		_alpm_log(PM_LOG_DEBUG, _("successfully downloaded %s"), filename);
-		alpm_list_free(files);
-		alpm_list_free(servers);
+	/* TODO this seems like needless complexity just to download one file */
+	alpm_list_t *servers = alpm_list_add(NULL, server);
+	alpm_list_t *files = alpm_list_add(NULL, filename);
+
+	/* download the file */
+	if(_alpm_downloadfiles(servers, cachedir, files, NULL, 0)) {
+		_alpm_log(PM_LOG_WARNING, _("failed to download %s\n"), url);
+		return(NULL);
 	}
-
+	_alpm_log(PM_LOG_DEBUG, "successfully downloaded %s\n", filename);
+	alpm_list_free(files);
+	alpm_list_free(servers);
 	_alpm_server_free(server);
 
-	/* return the target with the raw filename, no URL */
-	return(filename);
+	/* we should be able to find the file the second time around */
+	filepath = _alpm_filecache_find(filename);
+	return(filepath);
 }
 
 /* vim: set ts=2 sw=2 noet: */
