@@ -238,7 +238,7 @@ int _alpm_sync_sysupgrade(pmtrans_t *trans,
 		}
 
 		/* compare versions and see if we need to upgrade */
-		if(alpm_pkg_compare_versions(local, spkg)) {
+		if(_alpm_pkg_compare_versions(local, spkg)) {
 			_alpm_log(PM_LOG_DEBUG, "%s elected for upgrade (%s => %s)\n",
 					alpm_pkg_get_name(local), alpm_pkg_get_version(local),
 					alpm_pkg_get_version(spkg));
@@ -301,16 +301,7 @@ int _alpm_sync_addtarget(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sy
 				repo_found = 1;
 				spkg = _alpm_db_get_pkgfromcache(db, targ);
 				if(spkg == NULL) {
-					/* Search provides */
-					_alpm_log(PM_LOG_DEBUG, "target '%s' not found in db '%s' -- looking for provisions\n", targ, db->treename);
-					alpm_list_t *p = _alpm_db_whatprovides(db, targ);
-					if(!p) {
-						RET_ERR(PM_ERR_PKG_NOT_FOUND, -1);
-					}
-					spkg = (pmpkg_t *) p->data;
-					_alpm_log(PM_LOG_DEBUG, "found '%s' as a provision for '%s'\n",
-							alpm_pkg_get_name(spkg), targ);
-					alpm_list_free(p);
+					RET_ERR(PM_ERR_PKG_NOT_FOUND, -1);
 				}
 			}
 		}
@@ -325,25 +316,10 @@ int _alpm_sync_addtarget(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sy
 			spkg = _alpm_db_get_pkgfromcache(db, targ);
 		}
 		if(spkg == NULL) {
-			/* Search provides */
-			_alpm_log(PM_LOG_DEBUG, "target '%s' not found -- looking for provisions\n", targ);
-			for(j = dbs_sync; j && !spkg; j = j->next) {
-				pmdb_t *db = j->data;
-				alpm_list_t *p = _alpm_db_whatprovides(db, targ);
-				if(p) {
-					spkg = (pmpkg_t *) p->data;
-					_alpm_log(PM_LOG_DEBUG, "found '%s' as a provision for '%s' in db '%s'\n",
-							alpm_pkg_get_name(spkg), targ, db->treename);
-					alpm_list_free(p);
-				}
-			}
+			RET_ERR(PM_ERR_PKG_NOT_FOUND, -1);
 		}
 	}
 
-	if(spkg == NULL) {
-		RET_ERR(PM_ERR_PKG_NOT_FOUND, -1);
-	}
-
 	if(_alpm_pkg_should_ignore(spkg)) {
 		int resp;
 		QUESTION(trans, PM_TRANS_CONV_INSTALL_IGNOREPKG, spkg, NULL, NULL, &resp);
@@ -354,7 +330,7 @@ int _alpm_sync_addtarget(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sy
 
 	local = _alpm_db_get_pkgfromcache(db_local, alpm_pkg_get_name(spkg));
 	if(local) {
-		if(alpm_pkg_compare_versions(local, spkg) == 0) {
+		if(_alpm_pkg_compare_versions(local, spkg) == 0) {
 			/* spkg is NOT an upgrade */
 			if(trans->flags & PM_TRANS_FLAG_NEEDED) {
 				_alpm_log(PM_LOG_WARNING, _("%s-%s is up to date -- skipping\n"),
