@@ -1,7 +1,7 @@
 /*
  *  pacman.c
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2002-2008 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -108,7 +108,7 @@ static void usage(int op, const char * const myname)
 			printf(_("  -o, --owns <file>    query the package that owns <file>\n"));
 			printf(_("  -p, --file <package> query a package file instead of the database\n"));
 			printf(_("  -s, --search <regex> search locally-installed packages for matching strings\n"));
-			printf(_("  -t, --orphans        list all packages not required by any package\n"));
+			printf(_("  -t, --unrequired     list all packages not required by any package\n"));
 			printf(_("  -u, --upgrades       list all packages that can be upgraded\n"));
 			printf(_("  -q, --quiet          show less information for query and search\n"));
 		} else if(op == PM_OP_SYNC) {
@@ -151,7 +151,7 @@ static void version(void)
 {
 	printf("\n");
 	printf(" .--.                  Pacman v%s - libalpm v%s\n", PACKAGE_VERSION, LIB_VERSION);
-	printf("/ _.-' .-.  .-.  .-.   Copyright (C) 2002-2007 Judd Vinet <jvinet@zeroflux.org>\n");
+	printf("/ _.-' .-.  .-.  .-.   Copyright (C) 2002-2008 Judd Vinet <jvinet@zeroflux.org>\n");
 	printf("\\  '-. '-'  '-'  '-'\n");
 	printf(" '--'\n");
 	printf(_("                       This program may be freely redistributed under\n"
@@ -179,14 +179,12 @@ static void localize(void)
  */
 static void setuseragent(void)
 {
-	const char *pacman = "Pacman/" PACKAGE_VERSION;
-	const char *libalpm = "libalpm/" LIB_VERSION;
 	char agent[101];
 	struct utsname un;
 
 	uname(&un);
-	snprintf(agent, 100, "%s (%s %s %s; %s) %s", pacman, un.sysname,
-	         un.machine, un.release, setlocale(LC_MESSAGES, NULL), libalpm);
+	snprintf(agent, 100, "pacman/" PACKAGE_VERSION " (%s %s) libalpm/" LIB_VERSION,
+			un.sysname, un.machine);
 	setenv("HTTP_USER_AGENT", agent, 0);
 }
 
@@ -207,10 +205,11 @@ static void cleanup(int signum)
 		if(alpm_trans_interrupt() == 0) {
 			/* a transaction is being interrupted, don't exit pacman yet. */
 			return;
-		} else {
-			/* no commiting transaction, we can release it now and then exit pacman */
-			alpm_trans_release();
 		}
+		/* no commiting transaction, we can release it now and then exit pacman */
+		alpm_trans_release();
+		/* output a newline to be sure we clear any line we may be on */
+		printf("\n");
 	}
 
 	/* free alpm library resources */
@@ -325,7 +324,7 @@ static int parseargs(int argc, char *argv[])
 		{"root",       required_argument, 0, 'r'},
 		{"recursive",  no_argument,       0, 's'},
 		{"search",     no_argument,       0, 's'},
-		{"orphans",    no_argument,       0, 't'},
+		{"unrequired", no_argument,       0, 't'},
 		{"upgrades",   no_argument,       0, 'u'},
 		{"sysupgrade", no_argument,       0, 'u'},
 		{"verbose",    no_argument,       0, 'v'},
@@ -461,7 +460,7 @@ static int parseargs(int argc, char *argv[])
 				config->flags |= PM_TRANS_FLAG_RECURSE;
 				break;
 			case 't':
-				config->op_q_orphans = 1;
+				config->op_q_unrequired = 1;
 				break;
 			case 'u':
 				config->op_s_upgrade = 1;
@@ -607,8 +606,8 @@ static int _parseconfig(const char *file, const char *givensection,
 			 * follow the toupper() rules we may expect, e.g. tr_TR where i != I.
 			 */
 			upperkey = strtoupper(strdup(key));
-			if(section == NULL && (strcmp(key, "Include") == 0 || strcmp(upperkey, "INCLUDE") == 0)) {
-				pm_printf(PM_LOG_ERROR, _("config file %s, line %d: 'Include' directive must belong to a section.\n"),
+			if(section == NULL) {
+				pm_printf(PM_LOG_ERROR, _("config file %s, line %d: All directives must belong to a section.\n"),
 						file, linenum);
 				return(1);
 			}
